C $Header: /u/gcmpack/MITgcm/pkg/ptracers/ptracers_forcing_surf.F,v 1.11 2012/03/08 17:13:31 jmc Exp $
C $Name:  $

#include "PTRACERS_OPTIONS.h"

CBOP
C !ROUTINE: PTRACERS_FORCING_SURF

C !INTERFACE: ==========================================================
      SUBROUTINE PTRACERS_FORCING_SURF(
     I                            relaxForcingS,
     I                            bi, bj, iMin, iMax, jMin, jMax,
     I                            myTime,myIter,myThid )

C !DESCRIPTION:
C     Precomputes surface forcing term for pkg/ptracers.
C     Precomputation is needed because of non-local KPP transport term,
C     routine KPP_TRANSPORT_PTR.

C !USES: ===============================================================
      IMPLICIT NONE
#include "SIZE.h"
#include "EEPARAMS.h"
#include "PARAMS.h"
#include "GRID.h"
#include "SURFACE.h"
#include "DYNVARS.h"
#include "FFIELDS.h"
#include "PTRACERS_SIZE.h"
#include "PTRACERS_PARAMS.h"
#include "PTRACERS_START.h"
#include "PTRACERS_FIELDS.h"
C #include "EXF_PARAM.h"
#include "EXF_FIELDS.h"
#include "SEAICE_SIZE.h"
#include "SEAICE_PARAMS.h"
#include "SEAICE.h"



C !INPUT PARAMETERS: ===================================================
C  relaxForcingS        :: Salt forcing due to surface relaxation
C  bi,bj                :: tile indices
C  myTime               :: model time
C  myIter               :: time-step number
C  myThid               :: thread number
      _RL relaxForcingS(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      INTEGER bi, bj, iMin, iMax, jMin, jMax 
      _RL myTime
      INTEGER myIter
      INTEGER myThid

#ifdef ALLOW_PTRACERS

C !LOCAL VARIABLES: ====================================================
C  i,j                  :: loop indices
C  iTrc                 :: tracer index
C  ks                   :: surface level index
      INTEGER i, j
      INTEGER iTrc, ks, k !DV added k
      
      
     
      _RL add2EmP(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL epsil, cutoff, tmpVar
      _RL SchmidtNoAr(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL SchmidtNoKr(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL SchmidtNoN2(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL SchmidtNoXe(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL ArSat(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL KrSat(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL N2Sat(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL XeSat(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL TestSat(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL pisvel(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL wind(1-OLx:sNx+OLx,1-OLy:sNy+OLy,nSx,nSy)
      _RL KwexchPre(1-OLx:sNx+OLx,1-OLy:sNy+OLy,nSx,nSy)
      _RL KwexchAr(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL KwexchKr(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL KwexchN2(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL KwexchXe(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL KwexchTest(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL FluxAr(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL FluxKr(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL FluxN2(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL FluxXe(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL FluxTest(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL aTK
      _RL ArS
      _RL KrS
      _RL N2S
      _RL XeS
      _RL ttemp
      _RL stemp
      _RL arCnew
      _RL krCnew
      _RL n2Cnew
      _RL xeCnew
C      _RL PTR_Ar(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
C      _RL PTR_Kr(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
C      _RL PTR_N2(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
C      _RL PTR_Test(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
C Coefficients for calculating Schmidt number from Wanninkhof 2014 -DV    
      _RL aAr, bAr, cAr, dAr, eAr
      _RL aKr, bKr, cKr, dKr, eKr
      _RL aN2, bN2, cN2, dN2, eN2
      _RL aXe, bXe, cXe, dXe, eXe
C Coefficients for Setchénow relation from Weiss 1970, Weiss and Kyser 1978-DV
      _RL A1Ar, A2Ar, A3Ar, A4Ar, B1Ar, B2Ar, B3Ar
      _RL A1Kr, A2Kr, A3Kr, A4Kr, B1Kr, B2Kr, B3Kr
      _RL A1N2, A2N2, A3N2, A4N2, B1N2, B2N2, B3N2
      _RL A1Xe, A2Xe, A3Xe, B1Xe, B2Xe, B3Xe

CEOP
      
      aAr = 2078.1 _d 0
      bAr = -146.74 _d 0
      cAr = 5.6403 _d 0
      dAr = -0.11838 _d 0
      eAr = 0.0010148 _d 0
      
      aKr = 2252 _d 0
      bKr = -147.33 _d 0
      cKr = 5.1729 _d 0
      dKr = -0.10141 _d 0
      eKr = 0.00083242 _d 0
      
      aN2 = 2304.8 _d 0
      bN2 = -162.75 _d 0
      cN2 = 6.2557 _d 0
      dN2 = -0.13129 _d 0
      eN2 = 0.0011255 _d 0
      
      aXe = 2975.2 _d 0
      bXe = -201.6 _d 0
      cXe = 7.2057 _d 0
      dXe = -0.14287 _d 0
      eXe = 0.0011798 _d 0
      

      A1Ar = -173.5146 _d 0 
      A2Ar = 245.4510 _d 0
      A3Ar = 141.8222 _d 0
      A4Ar = -21.8020 _d 0
      B1Ar = -0.034474 _d 0
      B2Ar = 0.014934 _d 0
      B3Ar = -0.0017729 _d 0
      
      A1Kr = -109.9329 _d 0
      A2Kr = 149.8152 _d 0
      A3Kr = 72.8393 _d 0
      A4Kr = -9.9217 _d 0
      B1Kr = -0.006953 _d 0
      B2Kr = -0.004085 _d 0
      B3Kr = 0.0014759 _d 0
      
      A1N2 = -172.4965 _d 0
      A2N2 = 248.4262 _d 0
      A3N2 = 143.0738 _d 0
      A4N2 = -21.7120 _d 0
      B1N2 = -0.049781 _d 0
      B2N2 = 0.025018 _d 0
      B3N2 = -0.0034861 _d 0
C For Xe, Bunsen coeff. is in units of mol atm^-1 L^-1  -DV
      A1Xe = -7.48588 _d 0
      A2Xe = 5.08763 _d 0
      A3Xe = 4.22078 _d 0
      B1Xe = -0.00817791 _d 0
      B2Xe = -0.0120172 _d 0
      B3Xe = -0.006872 _d 0

      IF ( usingPCoords ) THEN
        ks = Nr
      ELSE
        ks = 1
      ENDIF

C====Here begins block pasted from dic_surfforcing.F -DV
       DO j=jMin,jMax
        DO i=iMin,iMax
C Compute AtmosP and Kwexch_Pre which are re-used for flux of O2

#ifdef USE_PLOAD
C Convert anomalous pressure pLoad (in Pa) from atmospheric model
C to total pressure (in Atm)
C Note: it is assumed the reference atmospheric pressure is 1Atm=1013mb
C       rather than the actual ref. pressure from Atm. model so that on
C       average AtmosP is about 1 Atm.
C                apressure(i,j,bi,bj)= 1. _d 0 + pLoad(i,j,bi,bj)/Pa2Atm
#endif

C Pre-compute part of exchange coefficient: pisvel*(1-fice)
C Schmidt number is accounted for later
              wind(i,j,bi,bj)= sqrt(wspeed(i,j,bi,bj)**2
     &                          + 6.5 _d 0*6.5 _d 0 )

              pisvel(i,j)= 0.337 _d 0 * wspeed(i,j,bi,bj)**2/3.6 _d 5
              KwexchPre(i,j,bi,bj) = pisvel(i,j)
     &                              * (1. _d 0 - AREA(i,j,bi,bj))

        ENDDO
       ENDDO
C=====Here block ends -DV


      
      

        DO j=jmin,jmax
          DO i=imin,imax
            IF (maskC(i,j,ks,bi,bj).NE.0.) THEN
              ttemp = theta(i,j,ks,bi,bj)
              stemp = salt(i,j,ks,bi,bj)
              aTK = 273.15 _d 0 + ttemp
C calculate SCHMIDT NOs. for Ar, Kr, N2, Xe  -DV
              SchmidtNoAr(i,j) =
     &            aAr
     &          + bAr * ttemp
     &          + cAr * ttemp*ttemp
     &          + dAr * ttemp*ttemp*ttemp
     &          + eAr * ttemp*ttemp*ttemp*ttemp
     
              SchmidtNoKr(i,j) =
     &            aKr
     &          + bKr * ttemp
     &          + cKr * ttemp*ttemp
     &          + dKr * ttemp*ttemp*ttemp
     &          + eKr * ttemp*ttemp*ttemp*ttemp
     
              SchmidtNoN2(i,j) =
     &            aN2
     &          + bN2 * ttemp
     &          + cN2 * ttemp*ttemp
     &          + dN2 * ttemp*ttemp*ttemp
     &          + eN2 * ttemp*ttemp*ttemp*ttemp
     
              SchmidtNoXe(i,j) =
     &            aXe
     &          + bXe * ttemp
     &          + cXe * ttemp*ttemp
     &          + dXe * ttemp*ttemp*ttemp
     &          + eXe * ttemp*ttemp*ttemp*ttemp
C Determine surface fluxes of Ar, Kr, N2, Xe
C exchange coeff accounting for ice cover and Schmidt no.


              KwexchAr(i,j) = KwexchPre(i,j,bi,bj)
     &                    / sqrt(SchmidtNoAr(i,j)/660.0 _d 0)
              KwexchKr(i,j) = KwexchPre(i,j,bi,bj)
     &                    / sqrt(SchmidtNoKr(i,j)/660.0 _d 0)
              KwexchN2(i,j) = KwexchPre(i,j,bi,bj)
     &                    / sqrt(SchmidtNoN2(i,j)/660.0 _d 0)
              KwexchXe(i,j) = KwexchPre(i,j,bi,bj)
     &                    / sqrt(SchmidtNoXe(i,j)/660.0 _d 0)
              KwexchTest(i,j) = KwexchPre(i,j,bi,bj)
     &                    / sqrt(SchmidtNoAr(i,j)/660.0 _d 0)
              
              arCnew = A1Ar + A2Ar * (100. _d 0 / aTK) 
     $           + A3Ar * log(aTK / 100. _d 0)
     $           + A4Ar * (aTK / 100. _d 0) + stemp * 
     $            (B1Ar + B2Ar * (aTK / 100. _d 0)
     $           + B3Ar * (aTK / 100. _d 0)**2)
     
              krCnew = A1Kr + A2Kr * (100. _d 0 / aTK) 
     $            + A3Kr * log(aTK / 100. _d 0)
     $           + A4Kr * (aTK / 100. _d 0) + stemp * 
     $           (B1Kr + B2Kr * (aTK / 100. _d 0) 
     $           + B3Kr * (aTK / 100. _d 0)**2)
     
              n2Cnew = A1N2 + A2N2 * (100. _d 0 / aTK)
     $           + A3N2 * log(aTK / 100. _d 0)
     $           + A4N2 * (aTK / 100. _d 0) + stemp * 
     $           (B1N2 + B2N2 * (aTK / 100. _d 0) 
     $           + B3N2 * (aTK / 100. _d 0)**2)
               
              XeCnew = A1Xe + A2Xe * (100. _d 0 / aTK)
     $           + A3Xe * log(aTK / 100. _d 0)
     $           + stemp *
     $           (B1Xe + B2Xe * (aTK / 100. _d 0)
     $           + B3Xe * (aTK / 100. _d 0)**2)
     
               
              arS = EXP(arCnew)
              krS = EXP(krCnew)
              n2S = EXP(n2Cnew)
              xeS = EXP(xeCnew)
C Convert from ml/l to mol/m^3 or mol/l to mol/m^3
              ArSat(i,j) = ArS/22391.6 _d 0 * 1. _d 3
              KrSat(i,j) = KrS/22391.6 _d 0 * 1. _d 3
              N2Sat(i,j) = N2S/22391.6 _d 0 * 1. _d 3
              XeSat(i,j) = XeS * 1. _d 3
              TestSat(i,j) = 0.02 _d 0
              
C Determine flux
              FluxAr(i,j) = KwexchAr(i,j)*
     &                     (1. _d 0 * ArSat(i,j)
     &                      - pTracer(i,j,ks,bi,bj,1))
              FluxKr(i,j) = KwexchKr(i,j)*
     &                     (1. _d 0 * KrSat(i,j)
     &                      - pTracer(i,j,ks,bi,bj,2))
              FluxN2(i,j) = KwexchN2(i,j)*
     &                     (1. _d 0 * N2Sat(i,j)
     &                      - pTracer(i,j,ks,bi,bj,3))
              FluxN2(i,j) = KwexchXe(i,j)*
     &                     (1. _d 0 * XeSat(i,j)
     &                      - pTracer(i,j,ks,bi,bj,4))
              FluxTest(i,j) = KwexchTest(i,j)*
     &                     (1. _d 0 * TestSat(i,j)
     &                      - pTracer(i,j,ks,bi,bj,5))
     
            ELSE
              FluxAr(i,j) = 0. _d 0
              FluxKr(i,j) = 0. _d 0
              FluxN2(i,j) = 0. _d 0
              FluxXe(i,j) = 0. _d 0
              FluxTest(i,j) = 0. _d 0
            ENDIF
            
           surfaceForcingPTr(i,j,bi,bj,1) = FluxAr(i,j)
C          pTracer(i,j,ks,bi,bj,1) = KwexchPre(i,j,bi,bj)
C          surfaceForcingPTr(i,j,bi,bj,1) = 0
     
           surfaceForcingPTr(i,j,bi,bj,2) = FluxKr(i,j)
C          pTracer(i,j,ks,bi,bj,2) = ttemp
C          surfaceForcingPTr(i,j,bi,bj,2) = 0
            
           surfaceForcingPTr(i,j,bi,bj,3) = FluxN2(i,j)
C          pTracer(i,j,ks,bi,bj,3) = wind(i,j,bi,bj)
C          surfaceForcingPTr(i,j,bi,bj,3) = 0
C          DO k=1,29 
C             pTracer(i,j,k,bi,bj,4) = pTracer(i,j,k,bi,bj,2)/
C     &             pTracer(i,j,k,bi,bj,1)
C          END DO
          surfaceForcingPTr(i,j,bi,bj,4) = FluxXe(i,j)   
          surfaceForcingPTr(i,j,bi,bj,5) = FluxTest(i,j)
C          surfaceForcingPTr(i,j,bi,bj,4) = apressure(i,j,bi,bj)

          END DO
        END DO
               
     
C Tracer numbers: 1—Ar, 2-Kr, 3-N2, 4-test (const sol) -DV       

        

C--   Option to convert Salt-relaxation into additional EmP contribution
      IF ( PTRACERS_addSrelax2EmP ) THEN
C-    here we assume that salt_EvPrRn = 0
C     set cutoff value to prevent too large additional EmP:
C       current limit is set to 0.1 CFL
        epsil = 1. _d -10
        cutoff = 0.1 _d 0 *drF(ks)/PTRACERS_dTLev(ks)
        IF ( ( (nonlinFreeSurf.GT.0 .OR. usingPCoords)
     &         .AND. useRealFreshWaterFlux )
     &     .OR.convertFW2Salt .EQ. -1. ) THEN
         DO j = jMin, jMax
          DO i = iMin, iMax
            tmpVar = MAX( salt(i,j,ks,bi,bj), epsil )
            add2EmP(i,j) = relaxForcingS(i,j)/tmpVar
            add2EmP(i,j) = rUnit2mass
     &                  *MAX( -cutoff, MIN( add2EmP(i,j), cutoff ) )
          ENDDO
         ENDDO
        ELSE
         DO j = jMin, jMax
          DO i = iMin, iMax
            add2EmP(i,j) = relaxForcingS(i,j)/convertFW2Salt
            add2EmP(i,j) = rUnit2mass
     &                  *MAX( -cutoff, MIN( add2EmP(i,j), cutoff ) )
          ENDDO
         ENDDO
        ENDIF
#ifdef ALLOW_DIAGNOSTICS
        IF ( useDiagnostics ) THEN
         CALL DIAGNOSTICS_FILL(add2EmP,'Add2EmP ',0,1,2,bi,bj,myThid)
        ENDIF
#endif /* ALLOW_DIAGNOSTICS */
      ELSE
        DO j = jMin, jMax
          DO i = iMin, iMax
            add2EmP(i,j) = 0. _d 0
          ENDDO
        ENDDO
      ENDIF
C-- end of "addEmP" setting

#ifdef EXACT_CONSERV
      IF ( (nonlinFreeSurf.GT.0 .OR. usingPCoords)
     &     .AND. useRealFreshWaterFlux ) THEN

       DO iTrc=1,PTRACERS_numInUse

c-  NonLin_FrSurf and RealFreshWaterFlux : PmEpR effectively changes
c   the water column height ; temp., salt, (tracer) flux associated
c   with this input/output of water is added here to the surface tendency.
c
         IF ( PTRACERS_StepFwd(iTrc) .AND.
     &        PTRACERS_EvPrRn(iTrc).NE.UNSET_RL ) THEN
          DO j = jMin, jMax
           DO i = iMin, iMax
             surfaceForcingPTr(i,j,bi,bj,iTrc) =
     &          surfaceForcingPTr(i,j,bi,bj,iTrc)
     &        + ( PmEpR(i,j,bi,bj) - add2EmP(i,j) )
     &          *( PTRACERS_EvPrRn(iTrc) - pTracer(i,j,ks,bi,bj,iTrc))
     &          *mass2rUnit
           ENDDO
          ENDDO
         ENDIF

       ENDDO

C---+----1----+----2----+----3----+----4----+----5----+----6----+----7-|--+----|
      ELSE
#else /* EXACT_CONSERV */
      IF (.TRUE.) THEN
#endif /* EXACT_CONSERV */

C--   EmPmR does not really affect the water column height (for tracer budget)
C     and is converted to a salt tendency.

       IF (convertFW2Salt .EQ. -1.) THEN
C-    use local surface tracer field to calculate forcing term:

        DO iTrc=1,PTRACERS_numInUse

         IF ( PTRACERS_StepFwd(iTrc) .AND.
     &        PTRACERS_EvPrRn(iTrc).NE.UNSET_RL ) THEN
C        account for Rain/Evap tracer content (PTRACERS_EvPrRn) using
C        local surface tracer
          DO j = jMin, jMax
           DO i = iMin, iMax
            surfaceForcingPTr(i,j,bi,bj,iTrc) =
     &          surfaceForcingPTr(i,j,bi,bj,iTrc)
     &        + ( EmPmR(i,j,bi,bj) + add2EmP(i,j) )
     &          *( pTracer(i,j,ks,bi,bj,iTrc) - PTRACERS_EvPrRn(iTrc))
     &          *mass2rUnit
           ENDDO
          ENDDO
         ENDIF

        ENDDO

       ELSE
C-    use uniform tracer value to calculate forcing term:

        DO iTrc=1,PTRACERS_numInUse

         IF ( PTRACERS_StepFwd(iTrc) .AND.
     &        PTRACERS_EvPrRn(iTrc).NE.UNSET_RL ) THEN
C     account for Rain/Evap tracer content (PTRACERS_EvPrRn) assuming uniform
C     surface tracer (=PTRACERS_ref)
          DO j = jMin, jMax
           DO i = iMin, iMax
            surfaceForcingPTr(i,j,bi,bj,iTrc) =
     &          surfaceForcingPTr(i,j,bi,bj,iTrc)
     &        + ( EmPmR(i,j,bi,bj) + add2EmP(i,j) )
     &            *( PTRACERS_ref(ks,iTrc) - PTRACERS_EvPrRn(iTrc) )
     &            *mass2rUnit
           ENDDO
          ENDDO
         ENDIF

        ENDDO

C-    end local-surface-tracer / uniform-value distinction
       ENDIF

      ENDIF

C---+----1----+----2----+----3----+----4----+----5----+----6----+----7-|--+----|

#endif /* ALLOW_PTRACERS */

      RETURN
      END
